@@ -11,16 +11,26 @@ from cloudinary.forms import CloudinaryJsFileField, cl_init_js_callbacks
 
 register = template.Library()
 
-@register.simple_tag
-def cloudinary_url(source, options_dict={}, **options):
+@register.simple_tag(takes_context=True)
+def cloudinary_url(context, source, options_dict={}, **options):
     options = dict(options_dict, **options)
+    try:
+        if context['request'].is_secure() and 'secure' not in options:
+            options['secure'] = True
+    except KeyError:
+        pass
     if not isinstance(source, CloudinaryImage):
         source = CloudinaryImage(source)
-    return source.build_url(**options) 
+    return source.build_url(**options)
 
-@register.simple_tag(name='cloudinary')
-def cloudinary_tag(image, options_dict={}, **options):
+@register.simple_tag(name='cloudinary', takes_context=True)
+def cloudinary_tag(context, image, options_dict={}, **options):
     options = dict(options_dict, **options)
+    try:
+        if context['request'].is_secure() and 'secure' not in options:
+            options['secure'] = True
+    except KeyError:
+        pass
     if not isinstance(image, CloudinaryImage):
         image = CloudinaryImage(image)
     return image.image(**options)
