@@ -6,6 +6,8 @@ from cloudinary.compat import (PY3, to_bytes, to_bytearray, to_string, unquote,
 """ @deprecated: use cloudinary.SHARED_CDN """
 SHARED_CDN = cloudinary.SHARED_CDN
 
+DEFAULT_RESPONSIVE_WIDTH_TRANSFORMATION = {"width": "auto", "crop": "limit"}
+
 def build_array(arg):
     if isinstance(arg, list):
         return arg
@@ -28,6 +30,7 @@ def encode_dict(arg):
         return arg
 
 def generate_transformation_string(**options):
+    responsive_width = options.pop("responsive_width", cloudinary.config().responsive_width)
     size = options.pop("size", None)
     if size:
         options["width"], options["height"] = size.split("x")
@@ -37,9 +40,9 @@ def generate_transformation_string(**options):
 
     crop = options.pop("crop", None)
     angle = ".".join([str(value) for value in build_array(options.pop("angle", None))])
-    no_html_sizes = has_layer or angle or crop == "fit" or crop == "limit"
+    no_html_sizes = has_layer or angle or crop == "fit" or crop == "limit" or responsive_width
 
-    if width and (float(width) < 1 or no_html_sizes):
+    if width and (width == "auto" or float(width) < 1 or no_html_sizes):
         del options["width"]
     if height and (float(height) < 1 or no_html_sizes):
         del options["height"]
@@ -71,19 +74,27 @@ def generate_transformation_string(**options):
         border = "%(width)spx_solid_%(color)s" % {"color": border.get("color", "black").replace("#", "rgb:"), "width": str(border.get("width", 2))}
 
     flags = ".".join(build_array(options.pop("flags", None)))
+    dpr = options.pop("dpr", cloudinary.config().dpr)
 
-    params = {"w": width, "h": height, "t": named_transformation, "b": background, "co": color, "e": effect, "c": crop, "a": angle, "bo": border, "fl": flags}
+    params = {"w": width, "h": height, "t": named_transformation, "b": background, "co": color, "e": effect, "c": crop, "a": angle, "bo": border, "fl": flags, "dpr": dpr}
     for param, option in {"q": "quality", "g": "gravity", "p": "prefix", "x": "x",
                           "y": "y", "r": "radius", "d": "default_image", "l": "overlay", "u": "underlay", "o": "opacity",
                           "f": "fetch_format", "pg": "page", "dn": "density", "dl": "delay", "cs": "color_space"}.items():
         params[param] = options.pop(option, None)
 
-    transformations = [param + "_" + str(value) for param, value in params.items() if (value or value == 0)]
-    transformations.sort()
-    transformation = ",".join(transformations)
+    transformation = ",".join(sorted([param + "_" + str(value) for param, value in params.items() if (value or value == 0)]))
     if "raw_transformation" in options:
         transformation = transformation + "," + options.pop("raw_transformation")
-    url = "/".join([trans for trans in base_transformations + [transformation] if trans])
+    transformations = base_transformations + [transformation]
+    if responsive_width:
+      responsive_width_transformation = cloudinary.config().responsive_width_transformation or DEFAULT_RESPONSIVE_WIDTH_TRANSFORMATION
+      transformations += [generate_transformation_string(**responsive_width_transformation)[0]]
+    url = "/".join([trans for trans in transformations if trans])
+
+    if width == "auto" or responsive_width:
+      options["responsive"] = True
+    if dpr == "auto":
+      options["hidpi"] = True
     return (url, options)
 
 def cleanup_params(params):
@@ -91,9 +102,9 @@ def cleanup_params(params):
 
 def sign_request(params, options):
     api_key = options.get("api_key", cloudinary.config().api_key)
-    if not api_key: raise Exception("Must supply api_key")
+    if not api_key: raise ValueError("Must supply api_key")
     api_secret = options.get("api_secret", cloudinary.config().api_secret)
-    if not api_secret: raise Exception("Must supply api_secret")
+    if not api_secret: raise ValueError("Must supply api_secret")
 
     params = cleanup_params(params)
     params["signature"] = api_sign_request(params, api_secret)
@@ -122,7 +133,7 @@ def cloudinary_url(source, **options):
 
     cloud_name = options.pop("cloud_name", cloudinary.config().cloud_name or None)
     if cloud_name == None:
-        raise Exception("Must supply cloud_name in tag or in configuration")
+        raise ValueError("Must supply cloud_name in tag or in configuration")
     secure = options.pop("secure", cloudinary.config().secure)
     private_cdn = options.pop("private_cdn", cloudinary.config().private_cdn)
     secure_distribution = options.pop("secure_distribution", cloudinary.config().secure_distribution)
@@ -180,7 +191,7 @@ def cloudinary_url(source, **options):
 def cloudinary_api_url(action = 'upload', **options):
     cloudinary_prefix = options.get("upload_prefix", cloudinary.config().upload_prefix) or "https://api.cloudinary.com"
     cloud_name = options.get("cloud_name", cloudinary.config().cloud_name)
-    if not cloud_name: raise Exception("Must supply cloud_name")
+    if not cloud_name: raise ValueError("Must supply cloud_name")
     resource_type = options.get("resource_type", "image")
     return "/".join([cloudinary_prefix, "v1_1", cloud_name, resource_type, action])
 
@@ -274,6 +285,7 @@ def build_upload_params(**options):
               "categorization": options.get("categorization"),
               "detection": options.get("detection"),
               "similarity_search": options.get("similarity_search"),
+              "background_removal": options.get("background_removal"),
               "upload_preset": options.get("upload_preset"),
               "phash": options.get("phash"),
               "auto_tagging": options.get("auto_tagging") and float(options.get("auto_tagging"))}
@@ -286,4 +298,4 @@ def __safe_value(v):
         else: 
             return "0"
     else:
-        return v
\ No newline at end of file
+        return v
